// Osoba.cpp: implementation of the COsoba class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "m26.h"
#include "Osoba.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

COsoba::COsoba()
{
	m_pmatka=NULL;
	m_potec=NULL;

	m_narozeni=0;
	m_umrti=0;

	m_pos=CPoint(50,50);
}

COsoba::COsoba(CString id,CString jmeno,CString prijmeni,TCHAR pohlavi,int narozeni,int umrti)
{
	m_id=id;
	m_jmeno=jmeno;
	m_prijmeni=prijmeni;
	m_pohlavi=pohlavi;
	m_narozeni=narozeni;
	m_umrti=umrti;

	m_pos=CPoint(0,0);

	m_firstdraw=TRUE;

	m_pmatka=NULL;
	m_potec=NULL;
}

COsoba::~COsoba()
{

}

void COsoba::Draw(CDC *pDC, BOOL sel)
{
	CString name;
	name.Format("%s %s",m_jmeno,m_prijmeni);

	if(m_firstdraw)
	{
		//nakreslime si nanecisto text a spocitame velikost:
		CRect rect(0,0,0,0);
		pDC->DrawText(name,&rect,DT_CALCRECT|DT_SINGLELINE);

		rect.OffsetRect(m_pos);

		m_rect=rect;
		m_rect.InflateRect(10,5);
	}

	//barvu obdelniku podle pohlavi

	if(sel)
	{
		pDC->FillSolidRect(m_rect,COLORREF(0xFFFF));
	}
	else
		pDC->FillSolidRect(m_rect,m_pohlavi=='M'?COLORREF(0xFF0000):COLORREF(0xFF));


	pDC->DrawText(name,&m_rect,DT_SINGLELINE);
}

//zjistuje, zda bod lezi v obdelniku zobrazene osoby
BOOL COsoba::IsRect(CPoint pt)
{
	return m_rect.PtInRect(pt);
}

//premisti zobrazovaci obdelnik na pozici pos
void COsoba::MoveTo(CPoint pos)
{
	int w=m_rect.Width();
	int h=m_rect.Height();

	m_pos.x=pos.x-w/2;
	m_pos.y=pos.y-h/2;

	m_rect=CRect(m_pos.x,m_pos.y,m_pos.x+w,m_pos.y+h);
}

//vrati zobrazovaci obdelnik
CRect COsoba::GetRect()
{
	return m_rect;
}

void COsoba::AddParent(COsoba *par)
{
	if(par->m_pohlavi=='M')
	{
		m_otec=par->m_id;
		m_potec=par;
	}
	else
	{
		m_matka=par->m_id;
		m_pmatka=par;
	}
}

void COsoba::DrawLinkParent(CDC *pDC)
{
	//nakreslime caru na matku a otce:
	CPoint pos;
	pos=m_pos;
	pos.Offset(m_rect.Width()/2,m_rect.Height()/2);

	if(m_pmatka)
	{
		pDC->MoveTo(pos);

		CPoint pos2=m_pmatka->m_pos;
		pos2.Offset(m_pmatka->m_rect.Width()/2,m_pmatka->m_rect.Height()/2);
		pDC->LineTo(pos2);
	}
	if(m_potec)
	{
		pDC->MoveTo(pos);

		CPoint pos2=m_potec->m_pos;
		pos2.Offset(m_potec->m_rect.Width()/2,m_potec->m_rect.Height()/2);
		pDC->LineTo(pos2);
	}
	

}

void COsoba::AddManzel(COsoba *obj)
{
	//kontrola, zda jiz nejsou manzele
	POSITION pos=m_manzele.GetHeadPosition();
	while(pos)
	{
		COsoba *os=(COsoba*)m_manzele.GetNext(pos);

		if(os==obj)
			return;
	}

	m_manzele.AddTail(obj);
}

//nakresli caru mezi manzely
void COsoba::DrawLinkManzel(CDC *pDC)
{
	CPoint pt;
	pt=m_pos;
	pt.Offset(m_rect.Width()/2,m_rect.Height()/2);

	CPen pen(PS_SOLID,1,COLORREF(0xFF)),*oldpen;

	oldpen=pDC->SelectObject(&pen);

	//projedeme seznam manzelu
	POSITION pos=m_manzele.GetHeadPosition();
	while(pos)
	{
		COsoba *obj=(COsoba*)m_manzele.GetNext(pos);

		pDC->MoveTo(pt);

		CPoint pos2=obj->m_pos;
		pos2.Offset(obj->m_rect.Width()/2,obj->m_rect.Height()/2);
		pDC->LineTo(pos2);
	}

	pDC->SelectObject(oldpen);

}

void COsoba::FreeManzele()
{
	m_manzele.RemoveAll();
}
