// Rodina.cpp: implementation of the CRodina class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "m26.h"

#include "osoba.h"
#include "Rodina.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRodina::CRodina()
{
	m_id=0;
}

CRodina::~CRodina()
{
	Free();
}

//uvolni pamet
void CRodina::Free()
{
	POSITION pos=m_list.GetHeadPosition();
	while(pos)
	{
		COsoba *obj=(COsoba*)m_list.GetNext(pos);

		delete obj;
	}

	m_id=0;

	m_list.RemoveAll();
}

//vygeneruje jedinecny identifikator
CString CRodina::GenerateID()
{
	CString id;
	BOOL unikat;
	do{
		id.Format("I%d",m_id);

		unikat=TRUE;
		POSITION pos=m_list.GetHeadPosition();
		while(pos)
		{
			COsoba *obj=(COsoba*)m_list.GetNext(pos);
			if(obj->m_id==id)
			{
				unikat=FALSE;
				break;
			}
		}

		m_id++;
	}while(!unikat);

	return id;
}

void CRodina::AddOsoba(COsoba *obj)
{
	m_list.AddTail(obj);
}

//zjisti, zda jsou duplicitni ID:
BOOL CRodina::CheckID()
{
	POSITION pos=m_list.GetHeadPosition();
	while(pos)
	{
		COsoba *obj=(COsoba*)m_list.GetNext(pos);
		
		POSITION pos2=m_list.GetHeadPosition();
		while(pos2)
		{
			COsoba *obj2=(COsoba*)m_list.GetNext(pos2);
			if(obj!=obj2)
				if(strcmp(obj->m_id,obj2->m_id)==0)
					return FALSE;
		}
	}

	return TRUE;
}


//vytvori vazby
void CRodina::UpdateLinks()
{
	//nejprve nulujeme
	POSITION pos=m_list.GetHeadPosition();
	while(pos)
	{
		COsoba *obj=(COsoba*)m_list.GetNext(pos);
		obj->FreeManzele();

		obj->m_pmatka=NULL;
		obj->m_potec=NULL;
	}

	pos=m_list.GetHeadPosition();
	while(pos)
	{
		COsoba *obj=(COsoba*)m_list.GetNext(pos);
		
		if(obj->m_matka.IsEmpty() && obj->m_otec.IsEmpty())
			continue;
		
		POSITION pos2=m_list.GetHeadPosition();
		while(pos2)
		{
			COsoba *obj2=(COsoba*)m_list.GetNext(pos2);

			if(strcmp(obj2->m_id,obj->m_matka)==0)
			{
				obj->m_pmatka=obj2;
			}
			else if(strcmp(obj2->m_id,obj->m_otec)==0)
			{
				obj->m_potec=obj2;
			}

			//pridame manzele:
			if(obj->m_pmatka && obj->m_potec)
			{
				obj->m_pmatka->AddManzel(obj->m_potec);
				obj->m_potec->AddManzel(obj->m_pmatka);
			}
		}
	}
}


//sestav strom
void CRodina::BuildTree(CRect view)
{
	int ymin=0,ymax=0;
	//najdeme minim a maxima
	POSITION pos=m_list.GetHeadPosition();
	while(pos!=NULL)
	{
		COsoba *obj=(COsoba*)m_list.GetNext(pos);
		if(obj->m_narozeni!=0)
		{
			if(obj->m_narozeni<ymin || ymin==0)
				ymin=obj->m_narozeni;
			if(obj->m_narozeni>ymax || ymax==0)
				ymax=obj->m_narozeni;
		}

	}

	if(view.Height()==0 || m_list.GetCount()==0 || ymax==ymin)
		return;

	int delta=view.Width()/m_list.GetCount();
	int ydelta=view.Height()/(ymax-ymin);
	int index=0;
	pos=m_list.GetHeadPosition();
	while(pos!=NULL)
	{
		COsoba *obj=(COsoba*)m_list.GetNext(pos);

		CPoint pt;
		pt.x=index*delta+15;

		index++;
		
		if(obj->m_narozeni==0)
		{
			pt.y=15;
		}
		else
		{
			pt.y=(int)((obj->m_narozeni-ymin)*ydelta)+15;
		}

		obj->m_pos=pt;

		TRACE("pos: %d, %d\n",obj->m_pos.x,obj->m_pos.y);

		/*if(!obj->m_pmatka && !obj->m_potec)
		{
			delta+=10;
			obj->m_pos=CPoint(delta+15,15);
			delta+=obj->m_rect.Width();

			//nasli jsme nejaky koren:
			//BuildSubTree(obj,view);
		}*/
	}
		
	TRACE("%d, %d\n",ymin,ymax);

}

void CRodina::UpdateManzele()
{
	//nejprve nulujeme
	POSITION pos=m_list.GetHeadPosition();
	while(pos)
	{
		COsoba *obj=(COsoba*)m_list.GetNext(pos);
		obj->FreeManzele();
	}

	//pak aktualizujeme
	pos=m_list.GetHeadPosition();
	while(pos)
	{
		COsoba *obj=(COsoba*)m_list.GetNext(pos);

		if(obj->m_pmatka && obj->m_potec)
		{
			obj->m_pmatka->AddManzel(obj->m_potec);
			obj->m_potec->AddManzel(obj->m_pmatka);
		}
	}
}

void CRodina::BuildSubTree(COsoba *parent, CRect view)
{
	POSITION pos=m_list.GetHeadPosition();
	int delta=0;
	while(pos!=NULL)
	{
		COsoba *obj=(COsoba*)m_list.GetNext(pos);

		obj->m_pos=parent->m_pos;
		obj->m_pos.Offset(delta,parent->m_rect.Height()+10);
		delta+=obj->m_rect.Width()+10;

		if(obj->m_pmatka==parent || obj->m_potec==parent)
		{
			BuildSubTree(obj,view);
		}
	}
}
